﻿#include "StdAfx.h"
#include "AgentWrapper.h"
#include "RxHelper.h"
#include "updateutil.h"
#include <ShellAPI.h>

#define DBOX(x)	MessageBox(::GetForegroundWindow(), x, _T("Debug!"), MB_OK | MB_ICONINFORMATION )
#define DTRACE	ATLTRACE


//////////////////////////////////////////////////////////////////////////



DWORD ExecuteWait(LPCTSTR szFileName, int nShow, LPCTSTR szDir, LPCTSTR szParameter)
{
	SHELLEXECUTEINFO shExIf;
	::ZeroMemory(&shExIf, sizeof(SHELLEXECUTEINFO));

	shExIf.cbSize = sizeof( SHELLEXECUTEINFO );
	shExIf.lpVerb = _T("open");
	shExIf.lpFile = szFileName;
	shExIf.nShow = nShow;
	shExIf.lpDirectory = szDir;
	shExIf.lpParameters = szParameter;
	shExIf.fMask = SEE_MASK_NOCLOSEPROCESS;
	ShellExecuteEx( &shExIf );

	DWORD dwExitCode ;
	while(1)
	{
		::GetExitCodeProcess( shExIf.hProcess, &dwExitCode );
		if( dwExitCode != STILL_ACTIVE )
		{
			break;
		}

		Sleep(10);
	}

	return dwExitCode;
}


DWORD ExecuteWait(LPCTSTR szPath, int nShow, LPCTSTR szParameter)
{
	CAtlString path(szPath);
	CAtlString fileName;
	CAtlString dirName;

	int posDir = path.ReverseFind(_T('\\'));

	fileName = path.Mid(posDir + 1);
	dirName = path.Mid(0, posDir);

	return ExecuteWait(fileName, nShow, dirName, szParameter);
}


void Execute(LPCTSTR szFileName, int nShow, LPCTSTR szDir, LPCTSTR szParameter, SHELLEXECUTEINFO *shExIf)
{
	ATLASSERT(shExIf);
	if ( shExIf == NULL )
	{
		return;
	}

	::ZeroMemory(shExIf, sizeof(SHELLEXECUTEINFO));

	shExIf->cbSize = sizeof( SHELLEXECUTEINFO );
	shExIf->lpVerb = _T("open");
	shExIf->lpFile = szFileName;
	shExIf->nShow = nShow;
	shExIf->lpDirectory = szDir;
	shExIf->lpParameters = szParameter;
	shExIf->fMask = SEE_MASK_NOCLOSEPROCESS;
	ShellExecuteEx( shExIf );
}


void Execute(LPCTSTR szPath, int nShow, LPCTSTR szParameter, SHELLEXECUTEINFO *shExIf)
{
	CAtlString path(szPath);
	CAtlString fileName;
	CAtlString dirName;

	int posDir = path.ReverseFind(_T('\\'));

	fileName = path.Mid(posDir + 1);
	dirName = path.Mid(0, posDir);

	Execute(fileName, nShow, dirName, szParameter, shExIf);
}


//////////////////////////////////////////////////////////////////////////

BOOL _GetInstallPath(LPTSTR szPath, const int cch)
{
	//fixme:
	StringCchCopy(szPath, cch, _T("c:\\Program Files\\faked"));
	return TRUE;
}

BOOL _GetLocalVersion(_VERSION& version)
{
	TCHAR szPath[MAX_PATH] = { NULL };
	if (_GetInstallPath(szPath, _countof(szPath)))
	{
		PathAppend(szPath, _T("version.xml"));
		if ( PathFileExists(szPath))
		{
			CRxDoc doc;	
			if ( doc.LoadFile(szPath) )
			{
				// <root>
				//   <version>1,0,0,1</version>
				// </root>
				CAtlString str = doc.NodeText(_T("root/version"));
				if ( str.GetLength() )
				{
					return GetVersionFromString( version, str);
				}
			}
		}
		else
		{
			ATLTRACE(_T("notfound=[%s]\n"), szPath);
		}
	}
	return FALSE;
}

BOOL _GetServerVersion(_VERSION& version)
{
	CRxDoc doc;
	if ( doc.LoadURL( APP_VERSION_URL ) )
	{
		CAtlString str = doc.NodeText(_T("root/version"));
		if ( str.GetLength() )
		{
			return GetVersionFromString( version, str);
		}
	}
	return FALSE;
}

//////////////////////////////////////////////////////////////////////////

CAgentWrapper::CAgentWrapper(void)
{
	m_UpdateStatus = UPDATE_DONE;
	m_nErrorCode = ERROR_NONE;
}

CAgentWrapper::~CAgentWrapper(void)
{
}



_VERSION CAgentWrapper::GetVersionLocal()
{
	DTRACE(_T("GetVersionLocal"));	
	_VERSION version ;
	_GetLocalVersion(version);
	return version;
}

_VERSION CAgentWrapper::GetVersionRecent()
{
	DTRACE(_T("GetVersionRecent"));
	_VERSION version ;
	 _GetServerVersion(version);
	return version;
}

BOOL CAgentWrapper::GetIsChanged()
{
	DTRACE(_T("GetIsChanged"));

	_VERSION version_Local, versoin_Server ;
	if ( _GetLocalVersion(version_Local)  &&
		 _GetServerVersion(versoin_Server) )
	{
		UPDATE update = CompareVersion(version_Local, versoin_Server);
		if ( update == update_miner || update == update_major )
		{
			return TRUE;
		}
	}

	return FALSE;
}

BOOL CAgentWrapper::GetIsInstalled()
{
	DTRACE(_T("GetIsInstalled"));
	TCHAR szPath[MAX_PATH] = { NULL };
	return GetUninstallerFilepath(szPath, _countof(szPath));
}

BOOL CAgentWrapper::DoUpgrade( bool bSilent )
{
	TCHAR szSetupPath[MAX_PATH] = { NULL };
	TCHAR szFilename[MAX_PATH] = { NULL };
	
	GetTempPath(_countof(szSetupPath), szSetupPath);
	StringCchPrintf(szFilename, _countof(szFilename), _T("%u-%s"), GetTickCount(), APP_SETUP_FILENAME);
	PathAppend(szSetupPath, szFilename);

	if ( URLDownloadToFile( NULL, APP_SETUP_URL, szSetupPath, 0, NULL) == S_OK )
	{
		if ( bSilent )
		{
			ExecuteWait( szSetupPath, SW_HIDE, _T("/S") );	
		}
		else
		{
			SHELLEXECUTEINFO info;
			Execute( szSetupPath, SW_NORMAL, NULL, &info);
		}

		DTRACE(_T("DoUpgrade"));
		return TRUE;
	}
	else
	{
		m_nErrorCode = ERROR_SETUPDOWNLOAD;
		return FALSE;
	}
}

BOOL CAgentWrapper::GetUninstallerFilepath( LPTSTR szUninstallerPath, const int cch )
{	
	DTRACE(_T("GetUninstallerFilepath"));

	if ( _GetInstallPath(szUninstallerPath, cch) )
	{
		PathAppend( szUninstallerPath, APP_UNINSTALLER_FILENAME);
		return PathFileExists(szUninstallerPath);
	}
	
	return FALSE;
}

int CAgentWrapper::GetUninstallerType( LPTSTR szUninsallParameter, const int cch )
{
	DTRACE(_T("GetUninstallerType"));
	return 0; //NSIS
	//return 1; //Custom
}

UPDATE_ERROR_CODE CAgentWrapper::GetErrorCode()
{
	DTRACE(_T("GetErrorCode"));
	return m_nErrorCode;
}

BOOL CAgentWrapper::GetInstallError( LPTSTR szError, const int cch )
{
	DTRACE(_T("GetInstallError"));
	if ( m_nErrorCode != ERROR_NONE )
	{
		StringCchCopy(szError, cch, GetErrorString(m_nErrorCode));
	}
	return ( m_nErrorCode != ERROR_NONE );
}

BOOL CAgentWrapper::GetUninstallError( LPTSTR szError, const int cch )
{
	DTRACE(_T("GetUninstallError"));
	if ( m_nErrorCode != ERROR_NONE )
	{
		StringCchCopy(szError, cch, GetErrorString(m_nErrorCode));
	}
	return ( m_nErrorCode != ERROR_NONE );
}

BOOL CAgentWrapper::GetAgreementShown()
{
	DTRACE(_T("GetAgreementShown"));
	return TRUE;
}

BOOL CAgentWrapper::GetAgreementIsShow()
{
	DTRACE(_T("GetAgreementIsShow"));
	return FALSE;
}

BOOL CAgentWrapper::GetAgreementURL( LPTSTR szAgreementURL, const int cch )
{
	DTRACE(_T("GetAgreementURL"));
	StringCchCopy(szAgreementURL, cch, APP_ADDITIONAL_LICENSEURL);
	return TRUE;
}

LPCTSTR CAgentWrapper::GetGuid()
{
	DTRACE(_T("GetGuid"));
	return AGENT_GUID;
}


//   에러 문구 정의  
const TCHAR ERR_MSG_WRONG_QUERY[] = _T("잘못된 업그레이드 요청입니다.");
const TCHAR ERR_MSG_UNKNOWN_ERROR[] = _T("알 수 없는 오류가 발생했습니다.");
const TCHAR ERR_MSG_CANT_COPY_FILE[] = _T("파일 복사에 실패했습니다.");
const TCHAR ERR_MSG_USER_CANCEL[] = _T("사용자에 의해 취소되었습니다.");
const TCHAR ERR_MSG_CANT_DOWN_INSTALL[] = _T("업그레이드 프로그램을 다운로드하는 데 실패했습니다.");
const TCHAR ERR_MSG_CANT_DOWN_DLLINFO[] = _T("업그레이드 정보를 가져오는 데 실패했습니다.");
const TCHAR ERR_MSG_WRONG_DLL_XML[] = _T("잘못된 업그레이드 정보 파일입니다.");
const TCHAR ERR_MSG_UNINSTALLING[] = _T("삭제 중에는 업데이트할 수 없습니다.");
const TCHAR ERR_MSG_UPDATING[] = _T("이미 업데이트가 진행되고 있습니다.");
const TCHAR ERROR_MSG_SETUPDOWNLOAD[] = _T("셋업파일을 다운로드할 수 없습니다.");

LPCTSTR GetErrorString(UPDATE_ERROR_CODE nErrCode)
{
	ATLASSERT( nErrCode != ERROR_NONE);

	if ( nErrCode == ERROR_UNKNOWN_ERROR )
	{
		return ERR_MSG_UNKNOWN_ERROR;
	}
	else if ( nErrCode == ERROR_UNINSTALLING )
	{
		return ERR_MSG_UNINSTALLING;
	}
	else if ( nErrCode == ERROR_UPDATING )
	{
		return ERR_MSG_UPDATING;
	}
	else if ( nErrCode == ERROR_WRONG_QUERY )
	{
		return ERR_MSG_WRONG_QUERY;
	}
	else if ( nErrCode == ERROR_SETUPDOWNLOAD)
	{
		return ERROR_MSG_SETUPDOWNLOAD;
	}

	return ERR_MSG_UNKNOWN_ERROR;
}
